
;;*****************************************************************************
;;  FILENAME: itoa.asm
;;  VERSION: 1.7
;;
;;  DESCRIPTION: MultiByte Integer-to-String Conversion Routines:
;;               itoa_i8,
;;               itoa_i16,
;;               itoa_i24,
;;               itoa_i32,
;;               itoa_i40
;;               itoa_format - nible (BCD) string to text string.
;;
;;  NOTE:  Do not call routines from different threads! 
;;
;;  Copyright (c) Ilya Mamontov 2007,2016. All Rights Reserved.
;;*****************************************************************************

include "m8c.inc"


area bss(RAM)
itoa_Integer::	BLK	5	;input Integer, destroyed
itoa_BCD::		BLK	7	;result nible (BCD) string, LS to MS digit
itoa_String::	BLK	16	;result text string
BCDdigits:		BLK	1
bitCount:		BLK	1
byteCount:		BLK	1
;for "format" procedure:
itoa_Length::	BLK	1	;input: required length of string (0 if the "space filling" not required)
						;output: real length of string (can be < required)

;------------------------------------------
area text(ROM,REL)


.SECTION
;-----------------------------------------------------------------------------
;  DESCRIPTION:	Converts 8, 16, 24, 32 and 40-bit Integer into nible (BCD) string.
;  ARGUMENTS:	"itoa_Integer" - input Integer (up to 5 bytes), destroyed
;  RETURNS:		"itoa_BCD" - nible (BCD) string, LS to MS digit, 14 digits, 7 bytes maximum
;-----------------------------------------------------------------------------
itoa_i8:: ;Result: 3 digits in "bcd" string
	mov [bitCount],8	;8 bit total
	mov [BCDdigits],2	;2 BCD bytes total (3 decimal digits)
	call m0
	jmp m8
itoa_i16:: ;Result: 5 digits in "bcd" string
	mov [bitCount],16	;16 bit total
	mov [BCDdigits],3	;3 BCD bytes total (5 decimal digits)
	call m0
	jmp m16
itoa_i24:: ;Result: 8 digits in "bcd" string
	mov [bitCount],24	;24 bit total
	mov [BCDdigits],4	;4 BCD bytes total (8 decimal digits)
	call m0
	jmp m24
itoa_i32:: ;Result: 10 digits in "bcd" string
	mov [bitCount],32	;32 bit total
	mov [BCDdigits],5	;5 BCD bytes total (10 decimal digits)
	call m0
	jmp m32
itoa_i40:: ;Result: 14 digits in "bcd" string
	mov [bitCount],40	;40 bit total
	mov [BCDdigits],7	;7 BCD bytes total (14 decimal digits)
	call m0
	jmp m40

;mov [itoa_Integer+0],255	;test!
;mov [itoa_Integer+1],255
;mov [itoa_Integer+2],255
;mov [itoa_Integer+3],255
;mov [itoa_Integer+4],255
m0:
	;--------------- initialization -----------------
	mov A,0
	mov [itoa_BCD+6],A	;clear "bcd" string
	mov [itoa_BCD+5],A
	mov [itoa_BCD+4],A
	mov [itoa_BCD+3],A
	mov [itoa_BCD+2],A
	mov [itoa_BCD+1],A
	mov [itoa_BCD+0],A
	ret
	;------------------- to BCD ---------------------
m40:
	rlc [itoa_Integer+0]	;for 40 bit Integer
	rlc [itoa_Integer+1]
	rlc [itoa_Integer+2]
	rlc [itoa_Integer+3]
	rlc [itoa_Integer+4]
	rlc [itoa_BCD+0]
	rlc [itoa_BCD+1]
	rlc [itoa_BCD+2]
	rlc [itoa_BCD+3]
	rlc [itoa_BCD+4]
	rlc [itoa_BCD+5]
	rlc [itoa_BCD+6]
	jmp m1 
m32:
	rlc [itoa_Integer+0]	;for 32 bit Integer
	rlc [itoa_Integer+1]
	rlc [itoa_Integer+2]
	rlc [itoa_Integer+3]
	rlc [itoa_BCD+0]
	rlc [itoa_BCD+1]
	rlc [itoa_BCD+2]
	rlc [itoa_BCD+3]
	rlc [itoa_BCD+4]
	jmp m1 
m24:
	rlc [itoa_Integer+0]	;for 24 bit Integer
	rlc [itoa_Integer+1]
	rlc [itoa_Integer+2]
	rlc [itoa_BCD+0]
	rlc [itoa_BCD+1]
	rlc [itoa_BCD+2]
	rlc [itoa_BCD+3]
	jmp m1 
m16:
	rlc [itoa_Integer+0]	;for 16 bit Integer
	rlc [itoa_Integer+1]
	rlc [itoa_BCD+0]
	rlc [itoa_BCD+1]
	rlc [itoa_BCD+2]
	jmp m1 
m8:
	rlc [itoa_Integer+0]	;for 8 bit Integer
	rlc [itoa_BCD+0]
	rlc [itoa_BCD+1]
m1:
	dec [bitCount]
	jz m9				;if BCD Conversion is over

	mov [byteCount],[BCDdigits]	;BCD digits (2, 3, 4, 5, 7) total
	mov X,itoa_BCD
m2:
	mov A,[X+0]
	add [X+0],03h
	tst [X+0],08h
	jnz m3
	mov [X+0],A
m3:
	mov A,[X+0]
	add [X+0],30h
	tst [X+0],80h
	jnz m4
	mov [X+0],A
m4:
	inc X
	dec [byteCount]
	jnz m2
	cmp [BCDdigits],2
	jz m8		;if 8-bit conversion
	cmp [BCDdigits],3
	jz m16		;if 16-bit conversion
	cmp [BCDdigits],4
	jz m24		;if 24-bit conversion
	cmp [BCDdigits],5
	jz m32		;if 32-bit conversion
	jmp m40	;if 40-bit conversion

m9:
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  DESCRIPTION:	Converts 14-nible BCD string into zero-terminated text string (13 chars maximum) 
;
;  ARGUMENTS:	"itoa_BCD" - nible (BCD) string, LS to MS digit, 7 bytes
;				"itoa_Length" 	=1..15  length of string (value is left justified, right spaces are added), 1 byte
;								=0      fixed length =13 (value is right justified, left unsigned zeros are added)
;  RETURNS:		"itoa_String" - result zero-truncated string, left-justified, rightly spaced
;				"itoa_Length" - number of written digits (1 byte)
;-----------------------------------------------------------------------------
itoa_format::
;mov [itoa_BCD+0],32h
;mov [itoa_BCD+1],54h
;mov [itoa_BCD+2],00h
;mov [itoa_BCD+3],00h
;mov [itoa_BCD+4],00h
;mov [itoa_BCD+5],00h
;mov [itoa_BCD+6],00h
	mov [byteCount],itoa_String	;load start addr of "itoa_String"
	mov X,6						;load start offset (MS digit) in bcd string, to process all the 14 digits, down to 0
	mov A, [itoa_Length]		;check "Length" parameter for 0
	jnz .t2						;"Length" is not 0, start with 13-th BCD digit (not 14-th)
		or  [itoa_BCD+6],40h	;"no skip" flag
		jmp .t2					;start with 13-th BCD digit (not 14-th)
.t0:;-- MS digit
	mov A,[X+itoa_BCD]
	asr A
	asr A
	asr A
	asr A
	and A,0Fh
	jnz .t1
		tst [itoa_BCD+6],F0h	;check "no skip" flag
		jz .t2					;skip to kill left zeros
	.t1:
	add A,'0'			;calculate char code
	mvi [byteCount],A	;write to text string
	or  [itoa_BCD+6],40h;set "no skip" flag

.t2:
	mov A,[X+itoa_BCD]
	and A,0Fh
	jnz .t3
		dec X					;check X (for "last digit")
		inc X					;
		jz .t3					;if X=0, write last digit (at least, one char must be present in string)
		tst [itoa_BCD+6],F0h	;check for "no skip" flag
		jz .t4					;skip to kill left zeros
	.t3:
	add A,'0'			;calculate char code
	mvi [byteCount],A	;write to text string
	or  [itoa_BCD+6],40h;set "no skip" flag
.t4:
	dec X				;next offset
	jnc .t0				;repeat if not end
	
	;-- calculate number of written digits
	mov A,[byteCount]
	sub A,itoa_String
	swap A,[itoa_Length]	;store number of written digits (to itoa_Length) and load required length
	;-- fill end of string by spaces (to keep the length)
	sub A,[itoa_Length]		;calculate number of extra spaces
	jc .t6					;if no extra spaces, nothing to add
	jz .t6					;
		mov X,A				;load number of extra spaces
	.t5:
		mov A,' '			;load "space"
		mvi [byteCount],A	;write "spaces" to text string
		dec X
		jnz .t5
.t6:;-- add EOL
	mov A,0
	mvi [byteCount],A	;write EOL
	ret

.ENDSECTION



