;main_proc.asm 
;Set of procedures for main.asm 

;Clk=32768*732=23986176 Hz with external watch crystal
;Clk=          24000000 Hz with external main oscillator


include "m8c.inc"
include "memory.inc"
include "PSoCAPI.inc"   ; PSoC API definitions for all User Modules
include "to_lcd_itoa.inc"

AREA InterruptRAM (RAM,REL,CON)
vstart::
c_old::		BLK	2	;old capture value
c_new::		BLK	2	;new capture value
c_CaptCnt::	BLK	3	;capture event counter, 2 bytes are used
c_Accum::	BLK	5	;Delta accumulator (in ticks of 24 MHz)
CaptCnt::	BLK	3	;result of c_CaptCnt
Accum::		BLK	5	;result of c_Accum
TimerCnt::	BLK	1	;Timer16_1 interrupt counter (24 MHz/65536=366.2 Hz)
TimerCntMax::BLK 1	;max value for "TimerCnt" variable (to set measurement time)
ready::		BLK	2	;flag: bit 3 = data are ready, bit 2 = reference cycle (not used)
systimer::	BLK	2	;free run system timer (366.2 Hz counter)
o:	BLK	1	;   
n:	BLK	1	;
c:	BLK	1	;


Result:		BLK	5	;result of measurement in Herz XXXX.
ResultF:	BLK	1	;result of measurement, fractional part, 0 to 10
noFreq:		BLK	1	;   : 1 -     ., 2  -   
StringPt::	BLK	1	;current pointer to char to transmit via UART
StringTX:	BLK	20	;string to transmit via UART
TM1::		BLK	5	;temp var for calculations
TM2::		BLK	5

key::   BLK	1	;key code
keyscan::   BLK	1	;keyscan code
keyp::  BLK	1	;keypress count flag
flag1:	BLK	1	;delay for typematic1
flag2:	BLK	1	;delay for typematic2


debug::		BLK 2	;
debug1::	BLK 2	;
phase::		BLK 2	;.. -     (    ), .  - 
modindex::	BLK	1	;  ( ) -  3 

ai::    BLK	1	;current address in led data array
am::    BLK	1	;scan mask
an::    BLK	12	;LED data array (6    PRT2DR and PRT2DM0) -       
				;   an+0:    an+2:    an+4:    an+6:    an+8:   an+10:
				;   1  -      mute    2 
				;  an+2 -     (   . )
aflag:: BLK	1	;  :	0b00000001 -  ""
				;						0b00000100 -  "1"-"2"     (     )
				;						0b11111100 -     (   73.2)

ci::    BLK	1	;        (  )
cv::    BLK	1	;    (0,1,2,3)

showmode::  BLK	1	;    (0 -   , 1 -  
showcount::	BLK	1	;    

;.    ,     
pitchVoice::	BLK	1	;  () 0...11
pitchRange::	BLK 1	; : 0 - , 1 - , 2 - , 3 - 
refFreq::		BLK 5	;   (      )

vend::		;label - end of variables
;------------- block of saved variables ------------
st::			BLK	2	;not used
sw::			BLK	2	;not used
su::			BLK	2	;not used
se::			BLK	2	;not used
sm::			BLK	2	;not used
sa::			BLK	2	;not used
ss2::			BLK	2	;not used
si1::			BLK	5	;not used
si2::			BLK	5	;not used
blockend::		BLK 1



AREA UserModules (ROM, REL)
;========================== procedures =============================

;-----------------------------------------------------------------------------
;  DESCRIPTION:	   -   (10)
;				 :
;				"freq" = "CaptCnt" * 120 000 000 (24MHz/2*10) * 128 (divider) / "Accum"
;
;  ARGUMENTS:	[CaptCnt] =   "":    "391"   100kHz, "3907"   1MHz,   "39063"   10MHz  (   1 )
;  				[Accum]   =     24:     2 400 000  -   24 000 000  (   0,1 - 1 )
;  RETURNS:		[Result] -   10 (5- ) + [ResultF] (  )
;				[noFreq] -   .: 1 -   .;  2 - .   
;-----------------------------------------------------------------------------
calculate_Result::
	;----- calculate result of measurement ------ 

include "math.inc"
	mov [noFreq],0						;    ., ,  , 
	;check for min allowed frequency
	mov A,[TimerCntMax+0]
	sub A,[CaptCnt+0] 
	mov A,0
	sbb A,[CaptCnt+1] 
	mov A,0
	sbb A,[CaptCnt+2] 
	jc .f_ok			;if CaptCnt > TimerCntMax
		;else prevent incorrect result - just write zeros
		math_load5	Result, 0, 0		;load "0" to integer part of result
		mov [ResultF],0					;load "0" to fractional part of result (  )
		mov [noFreq],1					;    .
		mov A,[CaptCnt+0]				;    . ( CaptCnt=0)
		or  A,[CaptCnt+1]
		or  A,[CaptCnt+2]
		jnz .end						; noFreq=1 (   .)
			mov [noFreq],2				; noFreq=2 (  .)
		jmp .end
.f_ok:
	;math_load5	TM1, 0, 24000000		;TM1="24 000 000"
	math_load5	TM1, 0, 120000000		;TM1="120 000 000"
	math_mul5x3	Result, TM1, CaptCnt	;Result=TM1*CaptCnt,     "CaptCnt" should be a 3-byte integer
	math_div5	Result, Accum			;Result=Result/Accum,    plus 5-byte "math_remainder"
	math_shl5	Result					;Result=Result*256
	math_shl5	math_remainder			;math_remainder=math_remainder*256
	math_mov5	TM1, math_remainder		;TM1=math_remainder*256
	math_div5	TM1, Accum				;TM1=TM1/Accum,    plus 5-byte "math_remainder"
	mov [Result],[TM1+0]				;combine with "remainder" byte

	;   
	;math_load5	TM1, 0, 937500			;     (  .) 10
	math_sub5	Result, refFreq, Result 	;Result=TM1-Result,    ""!
	
.end:
	ret

;-----------------------------------------------------------------------------
;  DESCRIPTION:	   
;				 :
;				"freq" = "CaptCnt" * 120 000 000 (24MHz/2*10) * 128 (divider) / "Accum"
;
;  ARGUMENTS:	[Result] -  ,   10  (5- )
;				[noFreq] -    (   .)
;
;  RETURNS:		[o] -   0...7	=-1 -     0  16 ,
;											= 0...7 -  ( 0   )
;											= 8 - 
;				[n] -  :	0...11   ("C"..."B")
;				[] -     0...255
;-----------------------------------------------------------------------------
Result2Pitch::
	f0: equ 20930-1	; ( 10)   ,   C7...B7  C8
	f1: equ 22175-1	;     ">="
	f2: equ 23493-1
	f3: equ 24890-1
	f4: equ 26370-1
	f5: equ 27938-1
	f6: equ 29600-1
	f7: equ 31360-1
	f8: equ 33224-1
	f9: equ 35200-1
	f10: equ 37293-1
	f11: equ 39511-1
	f12: equ 41860-1
;
	mov [o],7		;.
	mov [n],0		;.
	mov [TM1+0],[Result+0]
	mov [TM1+1],[Result+1]
		mov A,[Result+2];     16 
		or A,[Result+3]	;
		or A,[Result+4]	;
		jnz .exitmax		;     (c 8- )  . 
	jmp .enter

;          ( 7 )
.rep:
	dec [o]
	jc .exitmin	;       , ..   -1
	asl [TM1+0]	;*2 
	rlc [TM1+1]	;*2  
.enter:
	mov A,<f0	;,      0 ("C7")
	sub A,[TM1+0]
	mov A,>f0
	sbb A,[TM1+1]
	jnc .rep


;   -    ( 4 ) 
	mov A,<f8
	sub A,[TM1+0]
	mov A,>f8
	sbb A,[TM1+1]
	jnc .t8		; < f8 
		;8-9-10-11
		mov A,<f10
		sub A,[TM1+0]
		mov A,>f10
		sbb A,[TM1+1]
		jnc .t10		; < f10
			;10-11 (  12)
			mov A,<f11
			sub A,[TM1+0]
			mov A,>f11
			sbb A,[TM1+1]
			jnc .t11		; < f11
				;11 (  12)
				mov A,<f12
				sub A,[TM1+0]
				mov A,>f12
				sbb A,[TM1+1]
				jnc .t12		; < f12
					mov [n],11	;   12
					mov [o],8	; 
					jmp .exitmax;   (c 8- ) 
				.t12:mov [n],11	;   11
					 jmp .t99
			.t11:mov [n],10	;   10
				jmp .t99
		.t10:;8-9
			mov A,<f9
			sub A,[TM1+0]
			mov A,>f9
			sbb A,[TM1+1]
			jnc .t9			; < f9
				mov [n],9	;   9
				jmp .t99
			.t9:mov [n],8	;   8
				jmp .t99
	.t8:;0-1-2-3-4-5-6-7
	mov A,<f4
	sub A,[TM1+0]
	mov A,>f4
	sbb A,[TM1+1]
	jnc .t4			; < f4
		;4-5-6-7
		mov A,<f6
		sub A,[TM1+0]
		mov A,>f6
		sbb A,[TM1+1]
		jnc .t6			; < f6
			;6-7
			mov A,<f7
			sub A,[TM1+0]
			mov A,>f7
			sbb A,[TM1+1]
			jnc .t7			; < f7
				mov [n],7	;   7
				jmp .t99
			.t7:mov [n],6	;   6
				jmp .t99
		.t6:;4-5
			mov A,<f5
			sub A,[TM1+0]
			mov A,>f5
			sbb A,[TM1+1]
			jnc .t5			; < f5
				mov [n],5	;   5
				jmp .t99
			.t5:mov [n],4	;   4
				jmp .t99
	.t4:;0-1-2-3
		mov A,<f2
		sub A,[TM1+0]
		mov A,>f2
		sbb A,[TM1+1]
		jnc .t2			; < f2
			;2-3
			mov A,<f3
			sub A,[TM1+0]
			mov A,>f3
			sbb A,[TM1+1]
			jnc .t3			; < f3
				mov [n],3	;   3
				jmp .t99
			.t3:mov [n],2	;   2
				jmp .t99
		.t2:;0-1
			mov A,<f1
			sub A,[TM1+0]
			mov A,>f1
			sbb A,[TM1+1]
			jnc .t1			; < f1
				mov [n],1	;   1
				jmp .t99
			.t1:mov [n],0	;   0
				jmp .t99
	 .t99:	;  [n]    0...11
	 		;(  . [TM1]    )

;     0...255  ( = 256*(f-fn)/delta)
	;TM1=TM1-fn (2 ,  )
	asl [n]			; *2    (     !)
	mov A,[n]		;
	index fn+0		; fn  , . 
	sub [TM1+0],A	;TM1=TM1-fn, . 
	mov A,[n]		;
	index fn+1		; fn  , . 
	sbb [TM1+1],A	;TM1=TM1-fn, . 
	mov [TM1+2],0	;   
	mov [TM1+3],0
	mov [TM1+4],0
	;TM1=TM1*256
	math_shl5	TM1
	;TM2=delta
	mov A,[n]		;
	index delta+0	; delta  , . 
	mov [TM2+0],A
	mov A,[n]		;
	index delta+1	; delta  , . 
	mov [TM2+1],A
	mov [TM2+2],0	;   
	mov [TM2+3],0
	mov [TM2+4],0
	;TM1=TM1/TM2
	math_div5	TM1, TM2
	;    
	mov [c],[TM1+0]
	asr [n]			;  
;    c - [o], [n], [c]
	ret 

.exitmin:	;    ( 0- )
	;[o]=-1	   
	ret
.exitmax:	;   (c 8- )  . 
	mov [o],8	;  
	ret

.LITERAL	
;   	  0      1      2      3      4      5      6      7      8      9     10     11     12
fn:		DWL 20930, 22175, 23493, 24890, 26370, 27938, 29600, 31360, 33224, 35200, 37293, 39511, 41860
delta:	DWL  1245,  1318,  1397,  1480,  1568,  1662,  1760,  1864,  1976,  2093,  2218,  2349,  2489
.ENDLITERAL		

;-----------------------------------------------------------------------------
;  DESCRIPTION:	       
;				(    )
;				       ,  .
;  ARGUMENTS:	[o] -   0...7 	=-1 -     0  16 ,
;											= 0 - 0- ,  
;											= 8 - 
;				[n] -   0...11,    =13 -  
;				[] -     0...255
;				[Result] - (5-)  ,   10 (      .     )
;				[noFreq] -    : 1 -     ., 2  -   

;  RETURNS:		[an+0][an+1]   -   "1"
;				[an+10][an+11] -   "2"
;				[an+4][an+5] -   ""
;-----------------------------------------------------------------------------
ShowPitch::
	and [aflag],0b11111110	;  ,  ,  , 
;  .
	cmp [noFreq],1
	jz .set11			;     . -    [n]=11 ( "") 
	cmp [noFreq],2
	jnz .ok	
		;   .
		mov [o],0		;c  
		mov [n],13		;c  
		mov [c],0		; 
		jmp .main		;   
	.ok:

; -    ()     .
	tst [Result+4],0b10000000	;  
	jz .positive
		;  
		mov A,13				;          (    3   x10)
		add A,[Result+2]		;  12...0 ("12"    )
		;  ""
		;  2  -        FFh,  [n]=0 ( "")
		cmp [Result+3],FFh
		jnz .set0
		cmp [Result+4],FFh
		jnz .set0
		;  "12"   "11" (  12 +  )
		cmp A,12
		jnz .less
			mov A,13			;"12"   "13" -  
			jmp .noset0
		.less:
		jc .noset0
			.set0:				; >12, 
			mov A,0				;   [n]=0 ( "")
		.noset0:	
		mov [n],A				;     
		mov [c],0				; 
		mov [o],-1				;  "N"
		;   -  
		or reg[PGA_1_GAIN_CR1],0b00000011	;  PGA_1  AGND (011b)
		jmp .main				;   
	.positive:
		;   -  
		and reg[PGA_1_GAIN_CR1],0b11111001	;  PGA_1  Port Inputs (001b)
		;  
		cmp [o],-1				;    
		jnz .normal
			;    [o]=-1
			mov [n],13				;   ( )
			mov [c],0				; 
			or [aflag],0b00000001	;     
			jmp .main				;   
		.normal:
		 	;  ,  
			cmp [o],8				;  
			jnz .main				;  ,    
		 		;   ([o]=8)
				mov [n],[Result+2]		;        (  3   x10)
				;  
				;  2 
				mov A,[Result+3]
				or A, [Result+4]
				jnz .set11				;  ,  [n]=11 ( "")
				;  "11" (  12)
				cmp [n],12
				jc .noset11
					.set11:				; >=12, 
					mov [n],11			;   [n]=11 ( "")
				.noset11:	
				;
				mov [c],0				; 
				mov [o],8				;    ( =8,     )
				or [aflag],0b00000001	;     
				jmp .main				;   

.main:
ShowLEDs::
;-----------------------------------------------------------------------------
;  
;-----------------------------------------------------------------------------
;     [o], [n], [c] (  0  "aflag"    )
	;         
	cmp [n],11		;if((n==11)&(c>=192))	o++;   // . !    7
	jnz .y0
		cmp [c],192
		jc .y0
			inc [o]
	.y0:
	;    
	inc [o]			;  (-1...8) -> (0...9)
	mov A,[o]
	index octave_code0
	mov [an+4],A
	mov A,[o]
	index octave_code1
	mov [an+5],A

	;     
	; 1-  ("1")
	cmp [c],192
	jc .y1		 	; "" < 75  -    "1"
		inc [n]		;       (   12- -    . !)
	.y1:
	mov A,[n]
	index note_code0
	mov [an+0],A
	mov A,[n]
	index note_code1
	mov [an+1],A
	; 2-  ("2")
	cmp [c],192
	jnc .y2		 	; "" >= 75  -    "2"
	cmp [c],64
	jc .y2		 	; "" < 25  -    "2"
		inc [n]		;       (   12- -    . !)
	.y2:
	mov A,[n]
	index note_code0
	mov [an+10],A
	mov A,[n]
	index note_code1
	mov [an+11],A

	ret 
.LITERAL ;       
;        -1         0           1           2           3           4            5          6           7            8
octave_code0:DB 0b00000100, 0b00000000, 0b01000000, 0b00001000, 0b01000000, 0b00000001, 0b00000010, 0b00000100, 0b00001000, 0b00001000
octave_code1:DB 0b01000100, 0b00000000, 0b01000100, 0b01001000, 0b01001000, 0b10000001, 0b10000010, 0b10000100, 0b10001000, 0b10001000
;        0           1           2           3           4            5          6           7            8          9            10         11           12      13()
note_code0:	DB 0b00000001, 0b00100000, 0b00000010, 0b00100000, 0b00000100, 0b00100000, 0b00001000, 0b00100000, 0b00000001, 0b01000000, 0b00000010, 0b01000000, 0b00000001, 0b00000000
note_code1:	DB 0b00100001, 0b00100001, 0b00100010, 0b00100010, 0b00100100, 0b00100100, 0b00101000, 0b00101000, 0b01000001, 0b01000001, 0b01000010, 0b01000010, 0b00100001, 0b00000000
.ENDLITERAL		

;-----------------------------------------------------------------------------
;  DESCRIPTION:	    ShowLEDs (.    ShowPitch,   )
;				  -  
;  ARGUMENTS:	[showmode]=1,2,3 -   (    ,     ,  )
;  RETURNS:		[o] -   0...7 	=-1 -     0  16 ,
;											= 0 - 0- ,  
;											= 8 - 
;				[n] -   0...11,    =13 -  
;				[] = 0 (   )
;-----------------------------------------------------------------------------
ActionShow:
	dec [showcount]			;  ()  
	jnc .continue
		mov [showmode],0	;  -   
		ret
	
	.continue:			;  
	mov [o],0			;  ,  ,  , 	
	mov [n],13			;  ,    ,  , 
	;   (  )
	mov [c],0				; 
	and [aflag],0b11111110	;      

	cmp [showmode],1	;    
	jz showmode1
	cmp [showmode],2
	jz showmode2
	jmp showmode3
	
showmode1:	;   0...11
	tst [showcount],1	;   
	jz .led1off
		mov [n],[pitchVoice]	;  
	.led1off:
	ret

showmode2:	;   0...3
	tst [showcount],1	;   
	jz .led2off
		mov [o],[pitchRange];  
		inc [o]				;0...3    1...4
	.led2off:
	ret

showmode3:	;   -  " " ("showcount" = 12-1 = 11   )
	mov A,11
	sub A,[showcount]	
	mov [n],A			;   (A=0...11)
	;   
		sub A,3			;  
		jc .noLED		;   
		jnz .noZ
			dec A		;0   -1
			jmp .next
		.noZ: 
		cmp A,8
		jc .next
			.noLED:
			mov A,0		; 8...12     (  0)
		.next: 
	mov [o],A			;  	
	ret

;-----------------------------------------------------------------------------
;  DESCRIPTION:	Waits for new key code (an extra scan procedure is required).
;				While waits the "getkey_background" procedures is run.
;  ARGUMENTS:
;
;  RETURNS:		[key] - key code
;-----------------------------------------------------------------------------
getkey::
;waits for new key code
	scancode: equ an+2

.start:
	;wait for "ready" bit
	;----------------------------------------------------
	lcall getkey_background		;run background procedure
	;----------------------------------------------------
	tst [keyscan],0b10000000	;check "ready" bit (is set by external scan procedure)
	jz .start					;data not ready, wait

	;data in [keyscan] is ready (bits 3...0)
	;     approx. 18.3
	mov A,[keyscan]			;load "keyscan" code
	mov [keyscan],0			;clear "ready" bit and accumulated scan bits
	and A,0b00001111		;check scan bits 3...0
	jz .noKeypressed		;not keypressed ("keyscan"=0)
		;keypressed 
		mov [key],A			;return code

		mov A,[keyp]		;store old value of debounce counter
		mov [keyp],4		;set debounce counter=4
		inc [flag1]			;increase "flag1" (for typematic)
		sbb [flag1],0		;255 is maximum
		cmp A,0			;old value of debounce counter =0? 
		jz  .end		;yes, key realised	
			cmp [flag1],14		;keypress<14 cycl (~0,8 sec)?
			jc  .start			;yes, no typematic
			
			;else typematic (progressive speed)
			;uncomment next line if typematic is not required 
			jmp .start				;no typematic
			;test for specific keys
			;jnz .start				;if specific keys - no typematic
			
			;progressive speed of typematic (delay)
			mov A,[flag1]	;0..20..255
			add [flag2],A
			jnc .start
			jmp .end
.noKeypressed:;not keypressed 
	dec [keyp]			;decrease debounce counter
	adc [keyp],0		;0 is minimum
	jnz .start			;
		mov [key],0			;if 0 (not keypressed) clear code
		mov [flag1],0		;if 0 (not keypressed) clear typematic flag
	jmp .start

.end:
	mov [keyscan],0		;clear "keyscan" code for external scan procedure
	ret



;-----------------------------------------------------------------------------
;  DESCRIPTION:	Background cycle for "getkey" procedure
;				No loops here!
;  ARGUMENTS:   none
;-----------------------------------------------------------------------------
getkey_background::
	;UART: check RX for new code
	tst reg[RX8_1_CONTROL_REG],RX8_1_RX_REG_FULL	;Check if character is received
	jz .skip
	;-----------------------------   
		mov A,reg[RX8_1_RX_BUFFER_REG]				;character is ready, read code
		cmp A,'T'									;check char for "Terminate" command ('T'=84)
		jnz .check_S
			;if char='T'
			pop A	;correct stack pointer to simulate "return from getkey" 
			pop A
			ret					;return from "getkey"

		.check_S:
		cmp A,'S'									;check char for "Start" command ('S'=83)
		jnz .skip
			;char='S'
			pop A	;correct stack pointer to simulate "return from getkey" 
			pop A
			ret					;return from "getkey"
	.skip:

	;-----------------------------   
	mov A,[ready]		;check "ready" flag
	jz .dataNotReady
		;    
		lcall calculate_Result	;   (10)
		cmp [showmode],0		;     ...
		jz .noActionShow
			lcall ActionShow	;...     "showmode"
			lcall ShowLEDs		;    (.    ShowPitch)
			jmp .next
		.noActionShow:
			lcall Result2Pitch	;...     ,    ( )
			lcall ShowPitch		;   
		.next:
		mov [ready],0			;  
	.dataNotReady:
	ret

;end of main_proc.asm 
