;===============================================
;
; math.asm v1.5 (c)2004 Ilya Mamontov 
;
; Routines for 5-byte arithmetic
; Use macros in math.inc for easy usage
;
;===============================================
include "math.inc"
area bss(RAM)
math_aa1::			BLK	1	;addr of 1-st 5-byte operand
math_aa2::			BLK	1	;addr of 2-nd 5-byte operand
math_flag::			BLK	1	;flags
math_remainder::	BLK	5	;5 bytes of remainder
math_temp:			BLK	5	;5 bytes of math_temp variable
math_count:			BLK	1	;loop counter

area text(ROM,REL)
;------------------------------------------------------------------------------------
.SECTION
math__add5::
;===============================================
; (Un)signed 40-bit addition 
; [math_aa1]+[X]  ->  [math_aa2]
;
; math_aa1 and math_aa2 are incremented by 5 after operation
;===============================================
		mvi A,[math_aa1]	;[math_aa1]+[X] ->[math_aa2]
		add A,[X+0]
		mvi [math_aa2],A
		mvi A,[math_aa1]
		adc A,[X+1]
		mvi [math_aa2],A
		mvi A,[math_aa1]
		adc A,[X+2]
		mvi [math_aa2],A
		mvi A,[math_aa1]
		adc A,[X+3]
		mvi [math_aa2],A
		mvi A,[math_aa1]
		adc A,[X+4]
		mvi [math_aa2],A
		ret
.ENDSECTION
;------------------------------------------------------------------------------------
.SECTION
math__sub5::
;===============================================
; (Un)signed 40-bit subtraction  
; [math_aa1]-[X]  ->  [math_aa2]
;
; math_aa1 and math_aa2 are incremented by 5 after operation
;===============================================
		mvi A,[math_aa1]	; [math_aa1]-[X] ->[math_aa2]
		sub A,[X+0]
		mvi [math_aa2],A
		mvi A,[math_aa1]
		sbb A,[X+1]
		mvi [math_aa2],A
		mvi A,[math_aa1]
		sbb A,[X+2]
		mvi [math_aa2],A
		mvi A,[math_aa1]
		sbb A,[X+3]
		mvi [math_aa2],A
		mvi A,[math_aa1]
		sbb A,[X+4]
		mvi [math_aa2],A
		ret
.ENDSECTION
;------------------------------------------------------------------------------------
.SECTION
math__mul5x3::
;===============================================
; (Un)signed 40-bit by 24-bit multiplication 
; [math_aa1]*[math_aa2]  ->  [X]
;
; math_aa2 is incremented by 3 after operation
; (!)[math_aa2] should be 24-bit integer
; (!)Exception: math_aa1 = X is wrong combination
;===============================================
		mov [X+0],00h		;clear result
		mov [X+1],00h
		mov [X+2],00h
		mov [X+3],00h
		mov [X+4],00h

		mvi A,[math_aa2]	;copy [math_aa2]->[math_temp]
		mov [math_temp+0],A
		mvi A,[math_aa2]
		mov [math_temp+1],A
		mvi A,[math_aa2]
		mov [math_temp+2],A
		mov [math_count],24	;load count

mul5_3:	asl [X+0]			;shift left result
		rlc [X+1]
		rlc [X+2]
		rlc [X+3]
		rlc [X+4]

		rlc [math_temp+0]	;shift 24-bit math_temp (=math_aa2) 
		rlc [math_temp+1]
		rlc [math_temp+2]
		jnc mul5_4			;C=0

		mvi A,[math_aa1]	;else x=x+math_aa1
		add [X+0],A
		mvi A,[math_aa1]
		adc [X+1],A
		mvi A,[math_aa1]
		adc [X+2],A
		mvi A,[math_aa1]
		adc [X+3],A
		mvi A,[math_aa1]
		adc [X+4],A
		sub [math_aa1],5	;restore math_aa1		
mul5_4:
		dec [math_count]			
		jnz mul5_3			;repeat till 24 bits are done
		ret
.ENDSECTION
;------------------------------------------------------------------------------------
.SECTION
math__div5::	
;===============================================
; Unsigned 40-bit division  
; [X]/[math_aa1]  ->  [X].math_remainder
;
; (!)Variables [X] and [math_aa1] should be unsigned
; (!)No check on division by 0
;===============================================
		mov A,0					;clear remainder
		mov [math_remainder+0],A
		mov [math_remainder+1],A
		mov [math_remainder+2],A
		mov [math_remainder+3],A
		mov [math_remainder+4],A
		mov [math_count],40		;load loop count for 40-bit division
		
	d5_1:         
		;rotate left through dividend and remainder
		asl [X+0]	;clear LS bit of dividend
		rlc [X+1]				
		rlc [X+2]
		rlc [X+3]
		rlc [X+4]
		rlc [math_remainder+0]				
		rlc [math_remainder+1]
		rlc [math_remainder+2]				
		rlc [math_remainder+3]				
		rlc [math_remainder+4]				
		;make backup of remainder
		mov [math_temp+0],[math_remainder+0]
		mov [math_temp+1],[math_remainder+1]
		mov [math_temp+2],[math_remainder+2]
		mov [math_temp+3],[math_remainder+3]
		mov [math_temp+4],[math_remainder+4]
		;substract remainder from divisor
		mvi A,[math_aa1]
		sub [math_remainder+0],A
		mvi A,[math_aa1]
		sbb [math_remainder+1],A
		mvi A,[math_aa1]
		sbb [math_remainder+2],A
		mvi A,[math_aa1]
		sbb [math_remainder+3],A
		mvi A,[math_aa1]
		sbb [math_remainder+4],A

		jnc d5_2					
		mov [math_remainder+0],[math_temp+0];if result is negative
		mov [math_remainder+1],[math_temp+1];restore remainder from backup
		mov [math_remainder+2],[math_temp+2]
		mov [math_remainder+3],[math_temp+3]
		mov [math_remainder+4],[math_temp+4]
		jmp d5_3 
	d5_2:         
		or [X+0],01h	;if result is positive set LSB of dividend
	d5_3:
		sub [math_aa1],5;restore math_aa2
		dec [math_count]			
		jnz d5_1		;untill 40 bits are done
		ret
.ENDSECTION
;------------------------------------------------------------------------------------
.SECTION
math__abs5::
;===============================================
; Get 40-bit absolute value,
; bit 7 of "math_flag" is inverted if operand is negative (bits 0...6 are destroyed)
; abs([X])  ->  [X]
;===============================================
		mov A,[X+4]			;load MS byte of 40-bit operand
		xor [math_flag],A	;invert or not bit 7 of "math_count"
		rlc	A				;test bit 7 (sign)
		jc math__neg5		;if negative (C=1)
		ret
math__neg5::
;===============================================
; Get 40-bit negative value
; -[X]  ->  [X]
;===============================================
		mov A, 0xFF		;load 11111111 to A
        xor [X+0],A
        inc [X+0] 
        xor [X+1],A
        adc [X+1],0 
        xor [X+2],A
        adc [X+2],0 
        xor [X+3],A
        adc [X+3],0 
        xor [X+4],A
        adc [X+4],0 
		ret
.ENDSECTION
;------------------------------------------------------------------------------------
